------------------------------------------------------------------------------------------------
-- AISYSTEM
--
-- Copyright (C) 2012-Present. All rights reserved.
--
-- See LICENSE.TXT document for licensing information.
-------------------------------------------------------------------------------------------------
library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

library ieee_proposed;
use ieee_proposed.fixed_float_types.all;
use ieee_proposed.fixed_pkg.all;
use ieee_proposed.float_pkg.all;

library work;
use work.common.all;

library work;
use work.definition.all;

-- commander node role

entity command is
	port
    (
        reset               : in std_logic;
        clock               : in std_logic;
        -- inputs
		operation			: in operation_type;
		data_in				: in float(WE downto -WF);
		-- outputs
		data_out			: out float(WE downto -WF)
    );
end entity;


architecture rtl of command is
	signal dt				: float(WE downto -WF);
    signal t_step           : float(WE downto -WF);
	signal temp1			: float(WE downto -WF);
begin

	data_out <= temp1;

	operation_seq : process(reset, clock) is
		variable temp0		: float(WE downto -WF);
	begin
		if reset = '1' then
			dt <= (others => '0');
			t_step <= (others => '0');
			
			temp1 <= (others => '0');
		elsif rising_edge(clock) then
			case operation is 
					when OPR_SIMULATION_STEP =>				
						dt <= data_in;
						temp1 <= data_in;
					when OPR_SIMULATION_TIME =>				
						t_step <= data_in;
						temp1 <= data_in;
					when OPR_SIMULATION_ADVANCE =>
						temp0 := t_step;
						t_step <= temp0 + dt;
						temp1 <= t_step;
					when others =>
						null;
			end case;
		end if;
	end process;
	
end architecture;	  
