--------------------------------------------------------------------------
-- AISYSTEM
--
-- Copyright (C) 2012-Present. All rights reserved.
--
-- See LICENSE.TXT document for licensing information.
--------------------------------------------------------------------------
library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

-- common package for all applications

package common is
	-- version
	constant APP_SYSTEM_VERSION 	: integer := 1;
		
	-- nodes
    subtype node_role_type is natural range 0 to 2;
	
	constant NODE_ROLE_COMMANDER 	: node_role_type := 0;
	constant NODE_ROLE_SIMULATOR 	: node_role_type := 1;

	constant MAX_NODE_COUNT 		: integer := 1000; 		-- maximum number of nodes
	constant SIZE_NODE_COUNT		: integer := 4;		 	-- bits of MAX_NODE_COUNT
											   
	subtype node_type is integer range 0 to MAX_NODE_COUNT;

	constant NODE_ID_COMMANDER		: node_type := 1;		-- commander is first
	
	-- floating points
    constant WE						: integer := 8;
    constant WF 					: integer := 23;
    constant WEF 					: integer := 31;
    constant WEF1 					: integer := 32;
    constant WEF2 					: integer := 33;
	
	-- operation types
	constant MAX_OPERATION_TYPE		: integer := 999;
	constant SIZE_OPERATION_TYPE	: integer := 3;			-- bits of MAX_OPERATION_TYPE
	
	subtype operation_type is integer range 0 to MAX_OPERATION_TYPE;

	constant OPR_TYPE_NONE			: operation_type := 0;	-- no operation
	constant OPR_SIMULATION_ADVANCE	: operation_type := 1;	-- increment time step in commander
	constant OPR_SIMULATION_STEP	: operation_type := 2;	-- set dt in commander 
	constant OPR_SIMULATION_TIME	: operation_type := 3;	-- set initial time step in commander 
	constant OPR_SIMULATION_UNDEF	: operation_type := 4;	-- 
	constant OPR_NODE_ADVANCE		: operation_type := 5;	-- advance node simulation to specified time step in communicator or simulator

	-- miscellaneous
	subtype integer16 is integer range 0 to 65535;		

	constant CLOCK_FREQUENCY		: integer := 100_000_000;	-- 100 MHz
    constant BIT48 					: integer := 47;   
end package;

package body common is
end package body common;

