--------------------------------------------------------------------------
-- AISYSTEM
--
-- Copyright (C) 2012-Present. All rights reserved.
--
-- See LICENSE.TXT document for licensing information.
--------------------------------------------------------------------------
library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
						   
library work;
use work.common.all;
					   
-- definitions for simulator

package definition is
	-- computation phases
	subtype phase_type is integer range 0 to 1;

	constant COMP_PHASE_INITIALIZATION	: phase_type := 0;
	constant COMP_PHASE_COMPUTATION		: phase_type := 1;

	-- integration methods
    subtype method_type is natural range 0 to 5;
	
	constant INTG_RUNGE_KUTTA4      : method_type := 0;
	constant INTG_FORWARD_EULER     : method_type := 1;
	constant INTG_EXPONENTIAL_EULER : method_type := 2;
	constant INTG_BACKWARD_EULER    : method_type := 3;
	constant INTG_TRAPEZOID         : method_type := 4;
	constant INTG_CRANK_NICHOLSON   : method_type := 5;

	-- parameters (property values of an object)
	constant MAX_PARAMETER_COUNT	: integer := 30;
	constant PARAMETER_WIDTH		: integer := 32;		-- 32 bits
	
	subtype parameter_number_type is integer range 0 to MAX_PARAMETER_COUNT;
	subtype parameter_type is std_logic_vector(1 downto 0);
	
	type parameter_array is array (0 to MAX_PARAMETER_COUNT - 1) of parameter_type;

	-- parameter types
	constant PARAM_TYPE_NONE		: parameter_type := b"00";
	constant PARAM_TYPE_IN			: parameter_type := b"01";
	constant PARAM_TYPE_OUT		 	: parameter_type := b"10";
	constant PARAM_TYPE_INOUT		: parameter_type := b"11";

	-- computation order
	constant SIZE_COMPUTATION_ORDER	: integer := 4;			-- number of bits

	-- objects
	constant MAX_OBJECT_TYPE		: integer := 9999;		-- maximum number of object types
	

	subtype object_type is integer range 0 to MAX_OBJECT_TYPE;
	type computation_order_array is array (0 to SIZE_COMPUTATION_ORDER - 1) of object_type;
end package;

package body definition is
end package body definition;


