------------------------------------------------------------------------------------------------
-- AISYSTEM
--
-- Copyright (C) 2012-Present. All rights reserved.
--
-- See LICENSE.TXT document for licensing information.
-------------------------------------------------------------------------------------------------
library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

library ieee_proposed;
use ieee_proposed.fixed_float_types.all;
use ieee_proposed.fixed_pkg.all;
use ieee_proposed.float_pkg.all;

library work;
use work.common.all;

-- passive membrane channel
--
-- i_pas is a local current that should be added to the total membrane current but 
-- it doesn't contribute to any particular ionic concentration

entity object18 is  
    port
    (
        reset               : in std_logic;
        clock               : in std_logic;
        -- inputs
        phase           	: in phase_type;
        g                   : in float(WE downto -WF);
        v                   : in float(WE downto -WF);
        e                   : in float(WE downto -WF);
        -- outputs
        i_pas               : out float(WE downto -WF)
    );
end entity;

architecture rtl of object18 is
	signal step     : integer range 0 to 31 := 0;
	signal temp1 	: float(WE downto -WF);
begin
	passive_membrane : process (reset, clock) is   
    begin
        if reset = '1' then
            i_pas <= (others => '0');
            step <= 0;
        elsif rising_edge(clock) then
            case phase is
                when COMP_PHASE_INITIALIZATION =>
					i_pas <= (others => '0');
                when COMP_PHASE_COMPUTATION =>
                    step <= step + 1;
					case step is
						when 0 =>
		                    temp1 <= v - e;
						when 1 =>
		                    i_pas <= g * temp1;
                        when others => 
                            step <= 0;
					end case;
                when others =>
                    null;
            end case;
        end if;
    end process;
end architecture; 

