--------------------------------------------------------------------------
-- AISYSTEM
--
-- Copyright (C) 2012-Present. All rights reserved.
--
-- See LICENSE.TXT document for licensing information.
--------------------------------------------------------------------------
library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

library ieee_proposed;
use ieee_proposed.fixed_float_types.all;
use ieee_proposed.fixed_pkg.all;
use ieee_proposed.float_pkg.all;

library work;
use work.common.all;

-- linear synapse

entity object243 is
	generic
    (
		CLOCK_FREQUENCY	: integer := 100 -- [MHz]
    );
    port 
    (
        reset			: in std_logic;
        clock			: in std_logic;
        -- inputs
        phase           : in phase_type;
        v_m				: in float(WE downto -WF);
        weight         	: in float(WE downto -WF);
        -- outputs
        i_syn			: out float(WE downto -WF)
    );
end entity;


architecture rtl of object243 is
begin
	linear_synapse : process (reset, clock) is 
	begin
        if reset = '1' then
			i_syn <= (others => '0');
        elsif rising_edge(clock) then
            case phase is
                when COMP_PHASE_INITIALIZATION =>
					i_syn <= (others => '0');
                when COMP_PHASE_COMPUTATION =>
		            i_syn <= v_m * weight;
                when others =>
					null;
			end case;
        end if;
	end process;
end architecture;

