-------------------------------------------------------------------------------
-- Final rounding step for floating-point exponent and mantissa
--
-- Generics:
--   - wE [positive] : exponent bit-width
--   - wF [positive] : fraction bit-width
--
-- Ports:
--   - eX [in]  : exponent
--   - mX [in]  : mantissa (plus round and sticky bits)
--   - eR [out] : rounded exponent
--   - mR [out] : rounded mantissa
--
-- Rounding to nearest even. We may need to shift 1 bit right and increase the
-- exponent in case we reach the next binary interval.
-------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;

entity fp_round is
  generic ( wE : positive;
            wF : positive );
  port ( eX : in  std_logic_vector(wE downto 0);
         mX : in  std_logic_vector(wF+2 downto 0);
         eR : out std_logic_vector(wE downto 0);
         mR : out std_logic_vector(wF downto 0) );
end entity;

architecture arch of fp_round is
  signal round : std_logic;

  signal mR0 : std_logic_vector(wF+1 downto 0);
begin -------------------------------------------------------------------------

  round <= mX(1) and (mX(2) or mX(0));

  mR0 <= ("0" & mX(wF+2 downto 2)) + ((wF downto 0 => '0') & round);

  eR <= eX + ((wE-1 downto 0 => '0') & mR0(wF+1));

  mR <= (mR0(wF+1) or mR0(wF)) & mR0(wF-1 downto 0);

end architecture; -------------------------------------------------------------
