--------------------------------------------------------------------------
-- AISYSTEM
--
-- Copyright (C) 2012-Present. All rights reserved.
--
-- See LICENSE.TXT document for licensing information.
--------------------------------------------------------------------------
library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
use ieee.math_real.all;

library ieee_proposed;
use ieee_proposed.fixed_float_types.all;
use ieee_proposed.fixed_pkg.all;
use ieee_proposed.float_pkg.all;

library work;
use work.common.all;

entity command_tb is
end;

architecture bench of command_tb is
	component command
		port 
		(
			reset               : in std_logic;
			clock               : in std_logic;
			-- inputs
			operation			: in operation_type;
			data_in				: in float(WE downto -WF); 
			-- outputs
	        data_out            : out float(WE downto -WF)
		);
	end component;

	signal reset 				: std_logic;
	signal clock 				: std_logic;
	signal operation			: operation_type;
	signal data_in				: float(WE downto -WF);
	signal data_out				: float(WE downto -WF);
	signal is_stopped 			: boolean := false;
	-- test
	signal result 				: real;
begin
	uut : entity work.command(rtl)
	port map 
	(				 
		reset => reset,
		clock => clock,
		operation => operation,
		data_in	=> data_in,
		data_out => data_out
	);
	
	generation : process is
	begin
		while not is_stopped loop
			clock <= '0';
			wait for 5 ns;
			
			clock <= '1';
			wait for 5 ns;
		end loop;
		wait;
	end process;
	
	stimulus : process is
	begin				   
		-- initialize input signals
		reset <= '1';
		wait for 5 ns;
		reset <= '0';
		wait for 5 ns;

		operation <= OPR_TYPE_NONE;
		data_in <= (others => '0');
		wait until rising_edge(clock); -- cycle 1

		operation <= OPR_SIMULATION_STEP;		 
		data_in <= to_float(0.01, data_in);
		wait until rising_edge(clock); -- cycle 2

		result <= to_real(data_out);
		report "dt=" & real'image(result) severity note;

		operation <= OPR_SIMULATION_TIME;		 	  
		data_in <= to_float(434.12, data_in);
		wait until rising_edge(clock); 
		
		result <= to_real(data_out);
		report "set t_step=" & real'image(result) severity note;
		
		operation <= OPR_SIMULATION_ADVANCE;
		wait until rising_edge(clock); 

		result <= to_real(data_out);
		report "adv t_step=" & real'image(result) severity note;
		
		operation <= OPR_SIMULATION_ADVANCE;
		wait until rising_edge(clock); 

		result <= to_real(data_out);
		report "adv t_step=" & real'image(result) severity note;
		
		operation <= OPR_SIMULATION_ADVANCE;
		wait until rising_edge(clock); 
		
		result <= to_real(data_out);
		report "adv t_step=" & real'image(result) severity note;
		
		wait until rising_edge(clock); 
		
		result <= to_real(data_out);
		report "adv t_step=" & real'image(result) severity note;
		
		wait for 300 ns;
		
		is_stopped <= true;
		wait;
	end process;
end architecture;
