--------------------------------------------------------------------------
-- AISYSTEM
--
-- Copyright (C) 2012-Present. All rights reserved.
--
-- See LICENSE.TXT document for licensing information.
--------------------------------------------------------------------------
library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

library ieee_proposed;
use ieee_proposed.fixed_float_types.all;
use ieee_proposed.fixed_pkg.all;
use ieee_proposed.float_pkg.all;

library work;
use work.common.all;
   
-- test for object242   
   
entity test242 is
end;

architecture bench of test242 is

	component object242
		generic
		(
			CLK_FREQUENCY       : integer := 1
		);    
		port 
		(
			reset               : in std_logic;
			clock               : in std_logic;
			phase               : in phase_type;
			dt					: in unsigned(WEF downto 0); 
	        t_step              : in unsigned(BIT48 downto 0);
			absolute_refractory : in float(WE downto -WF);
			r_m                 : in float(WE downto -WF);
			c_m                 : in float(WE downto -WF);
			v_rest              : in float(WE downto -WF);
			i_syn               : in float(WE downto -WF);
			i_gap               : in float(WE downto -WF);
			i_inj               : in float(WE downto -WF);
			v_thr               : in float(WE downto -WF);
			v_m                 : out float(WE downto -WF)
		);
	end component;

	constant CLOCK_PERIOD		: time := 10 ns;

	signal reset				: std_logic;
	signal clock				: std_logic;
	signal dt					: unsigned(WEF downto 0);
	signal t_step              	: unsigned(BIT48 downto 0);
	signal absolute_refractory	: float(WE downto -WF);
	signal r_m					: float(WE downto -WF);
	signal c_m					: float(WE downto -WF);
	signal v_rest				: float(WE downto -WF);
	signal i_syn 				: float(WE downto -WF);
	signal i_gap 				: float(WE downto -WF);
  	signal i_inj 				: float(WE downto -WF);
	signal v_thr				: float(WE downto -WF);
  	signal v_m					: float(WE downto -WF);
	signal phase				: phase_type;
	signal v_m_out				: real;  
	signal is_stopped			: boolean;
begin
	-- values for generic parameters
	uut: object242 
		generic map 
		(
			CLK_FREQUENCY => 10
		)
		port map 
		(
			reset               => reset,
			clock               => clock,
			phase 				=> phase,
			dt                  => dt,
			t_step 			    => t_step,
			absolute_refractory => absolute_refractory,
			r_m                 => r_m,
			c_m                 => c_m,
			v_rest              => v_rest,
			i_syn               => i_syn,
			i_gap               => i_gap,
			i_inj               => i_inj,
			v_thr               => v_thr,
			v_m                 => v_m
		);

	v_m_out <= to_real(v_m);                         
	
	stimulus : process
	begin
		-- initialization
		dt <= to_unsigned(1 / 100, 32);
		t_step <= to_unsigned(1, 48);
		absolute_refractory <= to_float(2, 32);
		r_m <= to_float(10000, 32);
		c_m <= to_float(10, 32);
		v_rest <= to_float(-60, 32);
		i_syn <= to_float(0, 32);
		i_gap <= to_float(0, 32);
		i_inj <= to_float(300, 32);
		v_thr <= to_float(-40, 32);
		
		reset <= '1';
		wait for 5 ns;
		reset <= '0';
		wait for 5 ns;

		-- stimulus
        wait until rising_edge(clock);

		is_stopped <= true;
		wait;
	end process;

	clocking: process
	begin
		while not is_stopped loop
			clock <= '0', '1' after CLOCK_PERIOD / 2;
			wait for CLOCK_PERIOD;
		end loop;
		wait;
	end process;
end architecture;
  
