------------------------------------------------------------------------------------------------
-- AISYSTEM
--
-- Copyright (C) 2012-Present. All rights reserved.
--
-- See LICENSE.TXT document for licensing information.
-------------------------------------------------------------------------------------------------
library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

library ieee_proposed;
use ieee_proposed.fixed_float_types.all;
use ieee_proposed.fixed_pkg.all;
use ieee_proposed.float_pkg.all;

library work;
use work.common.all;

library work;
use work.definition.all;

-- * dendritic linear model that sums receiving input currents from all objects
*
* there is no back-propagation, the soma current is not computed
*
* V is membrane voltage and is computed based on the sum of all dendritic, 
* injection and synaptic currents; resistance is considered to be 1 (i.e. V = I)


PROP
	in:	V_init
		V_rest
		Vin
		Isyn
		Iinj
	out:
		v_out
INIT
	V_out = V_init
COMP
	v_out = V_rest + Vin + Isyn + Iinj




entity objectX is  
    port
    (
        reset               : in std_logic;
        clock               : in std_logic;
        -- inputs
        phase           	: in phase_type;
        prop1	            : in float(WE downto -WF);
        prop2	            : in float(WE downto -WF);
        prop3	            : in float(WE downto -WF);
        -- outputs
        prop4	            : out float(WE downto -WF);
    );
end entity;

architecture rtl of objectX is
	signal step     : integer range 0 to 31 := 0;
	signal temp1 	: float(WE downto -WF);
begin
	object_name : process (reset, clock) is   
    begin
        if reset = '1' then

			step <= 0;
        elsif rising_edge(clock) then
            case phase is
                when COMP_PHASE_INITIALIZATION =>
					
                when COMP_PHASE_COMPUTATION =>
                    step <= step + 1;
					case step is
						when 0 =>
							
						when 1 =>
				            
                        when others => 
                            step <= 0;
					end case;
                when others =>
					null;
			end case;
        end if;
    end process;
end architecture; 

