------------------------------------------------------------------------------------------------
-- AISYSTEM
--
-- Copyright (C) 2012-Present. All rights reserved.
--
-- See LICENSE.TXT document for licensing information.
-------------------------------------------------------------------------------------------------
library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

library ieee_proposed;
use ieee_proposed.fixed_float_types.all;
use ieee_proposed.fixed_pkg.all;
use ieee_proposed.float_pkg.all;

library work;
use work.common.all;

library work;
use work.definition.all;

-- * axon hillock 
*
* generates a single spike event with amplitude given by voltage membrane when the 
* input exceeds the specified threshold and there has not been a spike for at least 
* the time interval specified by absolute refractory
*
* 	Amplitude			spike event output amplitude
*	Vm				membrane voltage
*	Vthr				voltage threshold
*	Last_fired			last time an action potential was fired 
*	Absolute_refractory		minimum interval between spikes
--

PROP
	in:	
		last_fired
		absolute_refractory
		v_m
		v_rest
		V_thr
		amplitude
		state
	out:
		v_out
INIT
	Last_fired = -Absolute_refractory;
COMP
			if ((Last_fired + Absolute_refractory) >= (T_step + dt / 2.0))
			{
				/* refractory period */
				V_out = Vrest;
				State = 0;
			}
			else
			{
				if (V_m >= V_thr)
				{
					Last_fired = T_step;

					v_out = Amplitude;
					State = 1;
				}
				else
				{	
					v_out = Vrest;
					State = 0;
				}
			}
	





entity objectX is  
    port
    (
        reset               : in std_logic;
        clock               : in std_logic;
        -- inputs
        phase           	: in phase_type;
        prop1	            : in float(WE downto -WF);
        prop2	            : in float(WE downto -WF);
        prop3	            : in float(WE downto -WF);
        -- outputs
        prop4	            : out float(WE downto -WF);
    );
end entity;

architecture rtl of objectX is
	signal step     : integer range 0 to 31 := 0;
	signal temp1 	: float(WE downto -WF);
begin
	object_name : process (reset, clock) is   
    begin
        if reset = '1' then

			step <= 0;
        elsif rising_edge(clock) then
            case phase is
                when COMP_PHASE_INITIALIZATION =>
					
                when COMP_PHASE_COMPUTATION =>
                    step <= step + 1;
					case step is
						when 0 =>
							
						when 1 =>
				            
                        when others => 
                            step <= 0;
					end case;
                when others =>
					null;
			end case;
        end if;
    end process;
end architecture; 

